% un.m: Maximizes (minimizes negative) log likelihood function for
%         the unconstrained version of the Barro-Gordon model when
%         the natural rate of unemployment follows an AR(1) in
%         first differences.
%
%       When maximizing the log likelihood function, parameterizes
%         the Cholesky factorization of the covariance matrix
%         instead of the covariance matrix, instead of the
%         covariance matrix itself, in order to insure positive
%         definiteness. The log likelihood function with transformed
%         parameters is contained in unllfn.m.
%
%       When calculating standard errors, uses the original
%         parameterization. The log likelihood function without
%         transformed parameters is contained in unllfnse.m.
%
%       The untransformed parameter estimates are reported in
%         tstar = [ gam ppp ppu pup puu tpp1 tpu1 tup1 tuu1
%                   tpp2 tpu2 tup2 tuu2 sigp sigu sigpu]'.
%
%       Their standard errors are reported in sevec.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% load data

  global ut pit

  load u.dat;
  load def.dat;

  ut = u;
  pit = def;

% adjust sample size if necessary

%  T = length(ut);

%  ut = ut(41:T);

%  pit = pit(41:T);

% set starting values

%  gam = 0.18;
%  ppp = 0.91;
%  ppu = -0.25;
%  pup = 0.16;
%  puu = 0.54;
%  tpp1 = 0;
%  tpu1 = 0;
%  tup1 = 0;
%  tuu1 = 0;
%  tpp2 = 0;
%  tpu2 = 0;
%  tup2 = 0;
%  tuu2 = 0;
%  cholp = 0.31;
%  cholu = 0.26;
%  cholpu = -0.031;

  gam = 0.17;
  ppp = 0.92;
  ppu = -0.24;
  pup = 0.14;
  puu = 0.57;
  tpp1 = 0;
  tpu1 = 0;
  tup1 = 0;
  tuu1 = 0;
  tpp2 = 0;
  tpu2 = 0;
  tup2 = 0;
  tuu2 = 0;
  cholp = 0.28;
  cholu = 0.25;
  cholpu = -0.028;

  bigtheto = [ gam ppp ppu pup puu ...
               tpp1 tpu1 tup1 tuu1 ...
               tpp2 tpu2 tup2 tuu2 ...
               cholp cholu cholpu ]';

% maximize likelihood

  options(1) = 1;

  thetstar = fminu('unllfn',bigtheto,options);

% find standard errors

  gam = thetstar(1);
  ppp = thetstar(2);
  ppu = thetstar(3);
  pup = thetstar(4);
  puu = thetstar(5);
  tpp1 = thetstar(6);
  tpu1 = thetstar(7);
  tup1 = thetstar(8);
  tuu1 = thetstar(9);
  tpp2 = thetstar(10);
  tpu2 = thetstar(11);
  tup2 = thetstar(12);
  tuu2 = thetstar(13);

  cholsig = [ thetstar(14) 0 ; thetstar(16) thetstar(15) ];
  sig = cholsig*cholsig';

  sigp = sqrt(sig(1,1));
  sigu = sqrt(sig(2,2));
  sigpu = sig(1,2);

  tstar = [ gam ppp ppu pup puu ...
            tpp1 tpu1 tup1 tuu1 ...
            tpp2 tpu2 tup2 tuu2 ...
            sigp sigu sigpu ]';

  fstar = unllfnse(tstar);

  epsmat = 1e-6*eye(16);

  hessvec = zeros(16,1);

  for i = 1:16

    hessvec(i) = unllfnse(tstar+epsmat(:,i));

  end

  hessmat = zeros(16,16);

  for i = 1:16

    for j = 1:16

      hessmat(i,j) = (unllfnse(tstar+epsmat(:,i)+epsmat(:,j)) ...
                        -hessvec(i)-hessvec(j)+fstar)/1e-12;

    end

  end

  sevec = sqrt(diag(inv(hessmat)));